# delimit;
set more 1;
clear all;
set logtype text;

local year_start 2006;
local year_end 2012;

*local path_data 		/Volumes/FILES/Data/Nielsen;
local path_data 		/Volumes/data/Stata/States;
local upc_select 	= 1463; *1000,2000 or 1463;
local path_output 		/Volumes/FILES/Dropbox/MultiProductSearch/ScannerDecomposition/Output/29apr2015/moments_`upc_select';
local path_scratch_base 	/Volumes/FILES/Scratch;
local log_file			log_moments;

local use_varlist store_code_uc upc week_end units prmult price upc upc_ver_uc year fips_county_code fips_state_code dma_code;

*local state_list 1 4/6 8 9 12 13 17/27 29 32/34 36 37 39 41/42 45 47 48 51 53/55;
local state_list ${state_list_global};

local maxlag = 100;

local max_abs_z = 1.0;
local max_abs_y = 1.0;

local min_upc_by_store_week = 250;
local min_store_by_upc_week = 50;

quietly{;
	
foreach state of numlist `state_list' {;
	noi di "Working on state `state'";
	
	local path_output_geog `path_output'/State`state';	

	*create output directories;
	!mkdir -p `path_output_geog';
	!mkdir -p `path_output_geog'/store;
	!mkdir -p `path_output_geog'/storegood;
	
	*create scratch directory;
	local path_scratch `path_scratch_base'/`upc_select'/State`state';
	!mkdir -p `path_scratch';
	
	cap log close;
	log using `path_output_geog'/`log_file'.txt, replace;

	clear;
	save `path_scratch'/working_dataset.dta, replace emptyok;
	
	forvalues y = `year_start'/`year_end' {;		
		noi di " Working on year `y'";

		*load data;
		noi di " Loading in data";
		use `use_varlist' using `path_data'/`y'/AGG`upc_select'_`y'_`state'.dta, clear;
		
		*selections;
		noi di "  Doing selections";
		sort week_end upc;
		egen int nstore_by_upc_week = nvals(store_code_uc) , by(week_end upc);

		sort week_end store_code_uc;
		egen int nupc_by_store_week = nvals(upc) , by(week_end store_code_uc);

		keep if nstore_by_upc_week>=`min_store_by_upc_week';
		keep if nupc_by_store_week >= `min_upc_by_store_week';
		
		
		noi di "  Computing means by stores and goods";	
		*transaction prices;
		gen P = price/prmult;
		rename units q;
		gen Pq = P*q;
		drop price prmult;
		gen logP = log(P);

		*good-time means;
		bys week_end upc: egen mu_jt = mean(logP); 
		gen p_jst = logP - mu_jt;
		
		*estimate store effects;
		bys week_end store_code_uc: egen y_st = mean(p_jst);
		gen z_jst = p_jst - y_st;

		keep store_code_uc upc week_end z_jst y_st mu_jt;

		keep if abs(z_jst) < `max_abs_z';
		keep if abs(y_st) < `max_abs_y';
		
		append using `path_scratch'/working_dataset.dta;
		save `path_scratch'/working_dataset.dta, replace;
	};


	*store-good components;
	use `path_scratch'/working_dataset.dta, replace;
				
	*generate overall moments: z_jst;
	noi di " Generating overall moments for z_jst";
	sum z_jst;
	scalar cov_z_0 = r(Var);
	scalar nobs_z_0 = r(N);
	sort store_code_uc upc week_end;	
	forvalues d = 1/`maxlag'{;
		cap drop z_jstd;
		by store_code_uc upc: gen z_jstd = z_jst[_n-`d'];		
		qui corr z_jst z_jstd, cov;
		scalar cov_z_`d' = r(cov_12);
		scalar nobs_z_`d' = r(N);
	};
	
	forvalues m = 3/4{;
		gen z`m'_jst = z_jst^`m';
		sum z`m'_jst, meanonly;
		scalar mu`m'_z = r(mean);
		file open file`m' using `path_output_geog'/storegood/mu`m'.txt, text write replace;
		file write file`m' (mu`m'_z) _n;
		file close file`m'; 
	};
	
	*outsheet overall moments;
	clear;
	local ml1 = `maxlag'+1;
	set obs `ml1';
	gen lag = 0 if _n==1;
	gen cov = cov_z_0 if _n==1;
	gen nobs = nobs_z_0 if _n==1;
	forvalues d = 1/`maxlag'{;
		replace lag = `d' if _n==1+`d';
		replace cov = cov_z_`d' if _n==1+`d';
		replace nobs = nobs_z_`d' if _n==1+`d';
	};
	
	*save output in text files to be used in Fortran;
	sort lag;
	outsheet cov using `path_output_geog'/storegood/cov.txt, non replace;
	outsheet lag using `path_output_geog'/storegood/lag.txt, non replace;
	outsheet nobs using `path_output_geog'/storegood/nobs.txt, non replace;
	gen nmoments = _N;
	keep in 1;
	outsheet nmoments using `path_output_geog'/storegood/nmoments.txt, non replace;

	
	*store components;
	use `path_scratch'/working_dataset.dta, replace;

	sort store_code_uc week_end;		
	by store_code_uc week_end: keep if _n==1;
		
	*generate overall moments: y_st;
	noi di " Generating overall moments for y_st";	
	qui sum y_st;
	scalar cov_y_0 = r(Var);
	scalar nobs_y_0 = r(N);
	forvalues d = 1/`maxlag'{;
		cap drop y_std;
		by store_code_uc: gen y_std = y_st[_n-`d'];		
		qui corr y_st y_std, cov;
		scalar cov_y_`d' = r(cov_12);
		scalar nobs_y_`d' = r(N);
	};
	
	forvalues m = 3/4{;
		gen y`m'_st = y_st^`m';
		sum y`m'_st, meanonly;
		scalar mu`m'_y = r(mean);
		file open file`m' using `path_output_geog'/store/mu`m'.txt, text write replace;
		file write file`m' (mu`m'_y) _n;
		file close file`m'; 
	};
	
	*outsheet overall moments;
	clear;
	local ml1 = `maxlag'+1;
	set obs `ml1';
	gen lag = 0 if _n==1;
	gen cov = cov_y_0 if _n==1;
	gen nobs = nobs_y_0 if _n==1;
	forvalues d = 1/`maxlag'{;
		replace lag = `d' if _n==1+`d';
		replace cov = cov_y_`d' if _n==1+`d';
		replace nobs = nobs_y_`d' if _n==1+`d';
	};
	
	*save output in text files to be used in Fortran;
	sort lag;
	outsheet cov using `path_output_geog'/store/cov.txt, non replace;
	outsheet lag using `path_output_geog'/store/lag.txt, non replace;
	outsheet nobs using `path_output_geog'/store/nobs.txt, non replace;
	gen nmoments = _N;
	keep in 1;
	outsheet nmoments using `path_output_geog'/store/nmoments.txt, non replace;

	
	cap log close;
};

};


