#! /bin/sh                                                                                                             
#remember: cant have spaces between equals signs in assignments
#input (1) Debug/Release (2) run
# Set up environment variables for Mac and Intel Fortan 11.0

# source /opt/intel/composer_xe_2011_sp1/bin/compilervars.sh intel64

#Debug version
if [ "$1" == "Debug" ]; then
	echo "Compiling in " $1 "mode..."
	
	# Delete existing files
	rm *.o *.mod

	# Compile and link debug version: produces all debugging output, no optimizations, for use with Fx3
	ifort -m64 -g -debug all -implicitnone -save-temps -warn all -fp-stack-check -ftrapuv -check all -traceback -check noarg_temp_created \
			-gen-interfaces -warn interfaces -check arg_temp_created -check bounds\
		Parameters.f90 Globals.f90 Procedures.f90 \
		Estimation.f90 GetData.f90 MakeGuess.f90 \
		ObjFunction.f90 OutputResults.f90 \
		minpack.f90 \
		Main.f90 -o MainDebug.out

	# Create debug file with all script information
	dsymutil MainDebug.out

	# Create a startup script for Fx3
	#HOME=$PWD;	export HOME
	#if [ -f .Fx3init ]; then
	#    rm .Fx3init
	#fi

	#echo "addpath ./OtherCode \naddpath ./ObjectiveFunctions \nload Main.out" >> .Fx3init
	
	# Run Fx3
	#open -a Fx3
	
	# -g tells compiler to produce symbol info
	# -save-temps tells compiler not to delete intermediate files and puts them in working directory

	if [ "$2" == "run" ]; then
		./MainDebug.out
	fi

# Release version with openmp
elif [ "$1" == "Release" ]; then
		echo "Compiling in " $1 "mode..."
		
		ifort -m64 -O3 -openmp -traceback -mkl\
			Parameters.f90 Globals.f90 Procedures.f90 \
			Estimation.f90 GetData.f90 MakeGuess.f90 \
			ObjFunction.f90 OutputResults.f90 \
			minpack.f90 \
			Main.f90 -o MainRelease.out
			
	# -fast inclues-O3 plus some other optimization things
	# -openmp-link static forces the openmp libraries to be linked staticaly.
	# -O3 
	
	if [ "$2" == "run" ]; then
		./MainRelease.out
	fi


	fi

