SUBROUTINE Estimation

USE Parameters
USE Globals
USE Procedures

IMPLICIT NONE

EXTERNAL ObjFunction

!Local Variables
!----------------------------------
INTEGER									:: info,ip,io
REAL(8), DIMENSION(:), ALLOCATABLE	    :: Xguess, Xsol
REAL(8), DIMENSION(nmoments)	        :: Fsol


!Allocate variables
!------------------------------------
npar = 1 !AR
IF(OrderMA>0  .and. IncludeMA==1) npar = npar + OrderMA !MA
IF(IncludeFE==1) npar = npar + 1
npar = npar + 1 !veta: persistent shock
IF(IncludeMA==1) npar = npar + 1 !veps: transitory shock
IF(IncludeIIDSales==1 .or. IncludeUniformSales==1) THEN
	IF(IncludeIIDSales==1 .and. IncludeUniformSales==1 )  THEN
		write(*,*) " can't have both IID and Uniform sales"
		stop
	END IF

	npar = npar +2
	IF(EstimateVarianceSales==1) npar = npar+1
	IF(Estimate3rdMomentSales==1) npar = npar+1
	IF(Estimate4thMomentSales==1) npar = npar+1
END IF

IF(Estimate3rdMomentMA1==1) npar = npar+1
IF(Estimate3rdMomentAR1==1) npar = npar+1
IF(Estimate4thMomentMA1==1) npar = npar+1
IF(Estimate3rdMomentAR1==1) npar = npar+1


IF (ALLOCATED(Xguess)) DEALLOCATE(Xguess)
IF (ALLOCATED(Xsol)) DEALLOCATE(Xsol)
IF (ALLOCATED(Xfinal)) DEALLOCATE(Xfinal)
IF (ALLOCATED(Ffinal)) DEALLOCATE(Ffinal)

ALLOCATE(Xguess(npar),Xsol(npar),Xfinal(npar),Ffinal(nmoments))

!Construct guess
!-------------------
ip = 1

Xguess(ip) = logit(rhoguess)
ip = ip+1

IF(OrderMA>0  .and. IncludeMA==1) THEN
	DO io = 1,OrderMA
		Xguess(ip) = logit(thetaguess)
		ip = ip+1
	END DO
END IF	
	
IF(IncludeFE==1) THEN
	Xguess(ip) = vfeguess
	ip = ip+1	
END IF

Xguess(ip) = vetaguess
ip = ip+1

IF (IncludeMA==1) THEN
	Xguess(ip) = vepsguess
	ip = ip+1
END IF

IF(IncludeIIDSales==1 .or. IncludeUniformSales==1 )  THEN

	Xguess(ip) = musaleguess
	ip = ip+1
	Xguess(ip) = logit(saleprobguess)
	ip = ip+1
	
	IF(EstimateVarianceSales==1) THEN
		Xguess(ip) = log(vsaleguess)
		ip = ip+1
	END IF

	IF(Estimate3rdMomentSales==1) THEN
		Xguess(ip) = mu3saleguess
		ip = ip+1
	END IF

	IF(Estimate4thMomentSales==1) THEN
		Xguess(ip) = mu4saleguess
		ip = ip+1
	END IF
END IF

IF(Estimate3rdMomentMA1==1) THEN
	Xguess(ip) = mu3epsguess
	ip = ip+1
END IF

IF(Estimate3rdMomentAR1==1) THEN
	Xguess(ip) = mu3etaguess
	ip = ip+1
END IF

IF(Estimate4thMomentMA1==1) THEN
	Xguess(ip) = mu4epsguess
	ip = ip+1
END IF

IF(Estimate4thMomentAR1==1) THEN
	Xguess(ip) = mu4etaguess
	ip = ip+1
END IF


CALL ObjFunction (nmoments,npar,Xguess,Fsol,info)
CALL lmdif1 ( ObjFunction, nmoments, npar, Xguess, Fsol, 1.0e-6_8, info )
write(*,*) 'info is ',info
Xsol = Xguess
Xfinal = Xsol

!Extract solution
!--------------------
ip = 1

rho = logistic(Xsol(ip))
ip = ip+1

IF(OrderMA>0) THEN
	DO io = 1,OrderMA
		theta(io) = logistic(Xsol(ip))
		ip = ip+1
	END DO
ELSE 
	theta = 0.0
END IF	


IF(IncludeFE==1) THEN
	vfe = Xsol(ip)**2
	ip = ip+1
ELSE
	vfe = 0.0
END IF	

veta = Xsol(ip)**2
ip = ip+1

IF (IncludeMA==1) THEN
	veps = Xsol(ip)**2
	ip = ip+1
ELSE
	veps = 0.0
END IF

IF(IncludeIIDSales==1 .or. IncludeUniformSales==1 )  THEN
	
	musale = Xsol(ip)
	ip = ip+1
	saleprob = logistic(Xsol(ip))
	ip = ip+1

	IF(EstimateVarianceSales==1) THEN
		vsale = exp(Xsol(ip))
		mu2sale = vsale + musale**2
		ip = ip+1
	ELSE
		vsale = 0.0
		mu2sale = musale**2
	END IF

	IF(Estimate3rdMomentSales==1) THEN
		mu3sale = Xsol(ip)
		ip = ip+1
	ELSE
		mu3sale = musale**3
	END IF

	IF(Estimate4thMomentSales==1) THEN
		mu4sale = Xsol(ip)
		ip = ip+1
	ELSE
		mu4sale = 3.0*((mu2sale-musale**2)**2)
	END IF
	
ELSE
	musale = 0.0
	vsale = 0.0
	mu2sale = 0.0
	mu3sale = 0.0
	mu4sale = 0.0
	saleprob = 0.0
END IF


IF(Estimate3rdMomentMA1==1) THEN
	mu3eps = Xsol(ip)
	ip = ip+1
ELSE
	mu3eps = 0.0
END IF


IF(Estimate3rdMomentAR1==1) THEN
	mu3eta = Xsol(ip)
	ip = ip+1
ELSE
	mu3eta = 0.0
END IF

IF(Estimate4thMomentMA1==1) THEN
	mu4eps = Xsol(ip)**2
	ip = ip+1
ELSE
	mu4eps = 3.0*veps**2
END IF

IF(Estimate4thMomentAR1==1) THEN
	mu4eta = Xsol(ip)**2
	ip = ip+1
ELSE
	mu4eta = 3.0*veta**2
END IF

CALL ObjFunction (nmoments,npar,Xfinal,Ffinal,1)

END SUBROUTINE Estimation