SUBROUTINE ObjFunction (M,N,X,F,info)

USE Parameters
USE Globals
USE Procedures

IMPLICIT NONE

!Local Variables
!-----------------------
INTEGER, INTENT(IN) :: M						!number of equations / moment conditions
INTEGER, INTENT(IN)	:: N,info						!number of parameters
REAL(8), DIMENSION(N), INTENT(IN) :: X		!parameter vector: (rho, lveta, lveps)
REAL(8), DIMENSION(M), INTENT(OUT) :: F		!moment conditions
INTEGER				:: ia, it, im,ip, id, io
REAL(8)			:: lweight
REAL(8)		    :: lrho,lveta,lvfe,lveps,ltheta(6),lsaleprob,lmu3eps,lmu3eta,lmu4eps,lmu4eta,lmusale,lmu2sale,lmu3sale,lmu4sale,lnonsale,lvsale


!Extract parameters, incorporating transformations and identifying assumptions

ip = 1
lrho = logistic(X(ip))
ip = ip+1

ltheta = 0.0
IF(OrderMA>0 .and. IncludeMA==1) THEN	
	DO io = 1,OrderMA
		ltheta(io) = logistic(X(ip))
		ip = ip+1
	END DO
END IF

IF(IncludeFE==1) THEN
	lvfe = X(ip)**2
	ip = ip+ 1	
ELSE
	lvfe = 0.0
END IF

lveta = X(ip)**2
ip = ip+ 1	

IF(IncludeMA==1) THEN
	lveps = X(ip)**2
	ip = ip+ 1	
ELSE
	lveps = 0.0
END IF

IF(IncludeIIDSales==1 .or. IncludeUniformSales==1 )  THEN

	lmusale = X(ip)
	ip = ip+1
	lsaleprob = logistic(X(ip))
	ip = ip+1
	
	IF(EstimateVarianceSales==1) THEN
		lvsale = exp(X(ip))
		lmu2sale = lvsale + lmusale**2
		ip = ip+1
	ELSE
		lvsale = 0.0
		lmu2sale = lmusale**2
	END IF

	IF(Estimate3rdMomentSales==1) THEN
		lmu3sale = X(ip)
		ip = ip+1
	ELSE
		lmu3sale = lmusale**3
	END IF

	IF(Estimate4thMomentSales==1) THEN
		lmu4sale = X(ip)**2
		ip = ip+1
	ELSE
		lmu4sale = 3.0*((lmu2sale-lmusale**2)**2)
	END IF

ELSE
	lmusale = 0.0
	lvsale = 0.0
	lmu2sale = 0.0
	lmu3sale = 0.0
	lmu4sale = 0.0
	lsaleprob = 0.0
END IF

lnonsale = lsaleprob*lmusale/(1.0-lsaleprob)


IF(Estimate3rdMomentMA1==1) THEN
	lmu3eps = X(ip)
	ip = ip+1
ELSE
	lmu3eps = 0.0
END IF

IF(Estimate3rdMomentAR1==1) THEN
	lmu3eta = X(ip)
	ip = ip+1
ELSE
	lmu3eta = 0.0
END IF

IF(Estimate4thMomentMA1==1) THEN
	lmu4eps = X(ip)
	ip = ip+1
ELSE
	lmu4eps = 3.0*(lveps**2)
END IF

IF(Estimate4thMomentAR1==1) THEN
	lmu4eta = X(ip)
	ip = ip+1
ELSE
	lmu4eta = 3.0*(lveta**2)
END IF

!Calculate variance
!-------------------------------------------------

Vz = lveta/(1.0 - lrho**2)
Vu  = lveps
IF(OrderMA>0) THEN
	DO io = 1,OrderMA
		Vu  = Vu + (ltheta(io)**2)*lveps
	END DO
END IF

IF(IncludeUniformSales == 1) THEN
	Vs = (lnonsale**2) * ((1.0-lsaleprob)**2) &
		+ (2.0/3.0) * (lmu2sale + lmusale*lnonsale + lnonsale**2) * lsaleprob*(1.0-lsaleprob) &
		+ (1.0/3.0) * (2.0*lmu2sale + lmusale**2) * (lsaleprob**2)		
ELSE
	Vs = (1.0-lsaleprob)*(lnonsale**2) + lsaleprob*lmu2sale
END IF

Vy = Vz + Vu + lvfe + Vs

!Calculate covariances
!--------------------------------------------------
DO id = 1,nlag	
	
	COVy(id) = (lrho**id) * Vz + lvfe
	
	IF(IncludeMA==1 .and. OrderMA>0 .and. id<=OrderMA) THEN
		IF(id==OrderMA) THEN
			COVy(id) = COVy(id) + lveps*ltheta(OrderMA)		
		ELSE	
			DO io = 0,OrderMA-id-1
				COVy(id) = COVy(id) + lveps*ltheta(io+1)*ltheta(io+1+id)
			END DO
		END IF
	END IF
	
END DO

IF(IncludeUniformSales == 1) THEN
	COVy(1) = COVy(1) + (lnonsale**2) * ((1.0-lsaleprob)**3) &
					  + 0.5*(lnonsale**2 + lmusale*lnonsale) * lsaleprob * ((1.0-lsaleprob)**2) &
					  + ((2.0/3.0)*lnonsale*lmusale +(1.0/6.0)*(lnonsale**2+lmu2sale)) * lsaleprob * ((1.0-lsaleprob)**2) &
					  + (0.5*lnonsale*lmusale +(1.0/6.0)*lmu2sale + (1.0/3.0)*(lmusale**2)) * (lsaleprob**2) * (1.0-lsaleprob) &
					  + 0.5*(lnonsale*lmusale + lnonsale**2) * lsaleprob * ((1.0-lsaleprob)**2) &
					  + ((2.0/3.0)*lnonsale*lmusale +(1.0/6.0)*(lnonsale**2 + lmusale**2)) * (lsaleprob**2) * (1.0-lsaleprob) &
					  + (0.5*lnonsale*lmusale +(1.0/6.0)*lmu2sale + (1.0/3.0)*(lmusale**2)) * (lsaleprob**2) * (1.0-lsaleprob) &
					  + ((5.0/6.0)*(lmusale**2) + (1.0/6.0)*lmu2sale)* (lsaleprob**3)
END IF

!Calculate 3rd moments
E3z = lmu3eta/(1.0-lrho**3)
E3u = lmu3eps
IF(OrderMA>0) THEN
	DO io = 1,OrderMA
		E3u  = E3u + (ltheta(io)**3)*lmu3eps
	END DO
END IF
IF(IncludeUniformSales == 1) THEN
	E3s = (lnonsale**3)*(1.0-lsaleprob)**2  &
		 + 0.5*(lmu3sale + lmusale*(lnonsale**2) + lmu2sale*lnonsale + lnonsale**3) * lsaleprob * (1.0-lsaleprob) &
		 + 0.5*(lmu3sale +lmu2sale*lmusale) *(lsaleprob**2)
ELSE
	E3s = (1.0-lsaleprob)*(lnonsale**3) + lsaleprob*lmu3sale
END IF
E3y = E3z + E3u + E3s

!Calculate 4th moment
E4z = 6.0*(lrho**2)*lmu4eta/((1.0-lrho**2)*(1.0-lrho**4))
E4u = lmu4eps
IF(OrderMA>0) THEN
	DO io = 1,OrderMA
		E4u  = E4u + (ltheta(io)**4)*lmu4eps
	END DO
END IF
IF(IncludeUniformSales == 1) THEN
	E4s = (lnonsale**4)*(1.0-lsaleprob)**2  &
		 + (2.0/5.0)*(lmu4sale + lmu2sale*lnonsale**2 + lmu3sale*lnonsale + lmusale*(lnonsale**3) + lnonsale**4) * lsaleprob * (1.0-lsaleprob) &
		 + (1.0/5.0)*(2.0*lmu4sale +2.0*lmu3sale*lmusale + lmu2sale**2) *(lsaleprob**2)
ELSE
	E4s = (1.0-lsaleprob)*(lnonsale**4) + lsaleprob*lmu4sale
END IF

E4y = E4z + E4u + E4s + lvfe**2 +6.0*Vz*vu + 6.0*Vz*Vs + 6.0*Vz*lvfe + 6.0*Vu*Vs + 6.0*Vs*lvfe + 6.0*Vs*lvfe



!4. Fill in the theoretical/empirical moment differences corresponding to the data vector
!---------------------------------------------------- 
!$OMP PARALLEL DO PRIVATE(lweight)
DO im = 1, nlag+1
	lweight	= sqrt(real(nobs(im)))
	IF (IdentityWeight==1) lweight = 1.0

	IF (lag(im)==0) THEN
		F(im) = cov(im) - Vy
		dataVy = cov(im) 
	ELSE
		F(im) = cov(im) - COVy(lag(im))
		dataCOVy(lag(im)) = cov(im) 
	END IF
	
	!multiply by weight
	F(im) = F(im)*lweight
	
END DO
!$OMP END PARALLEL DO

im = nlag+1
IF(Match3rdMoment==1) THEN
	im = im+1
	lweight	= sqrt(real(nobs(im)))
	IF (IdentityWeight==1) lweight = 1.0
	
	F(im) = (cov(im)-E3y)*lweight
	dataE3y = cov(im)
END IF

IF(Match4thMoment==1) THEN
	im = im+1
	lweight	= sqrt(real(nobs(im)))
	IF (IdentityWeight==1) lweight = 1.0

	F(im) = (cov(im)-E4y)*lweight
	dataE4y = cov(im)
END IF


! write(*,*) sum(F(1:M)*F(1:M))


END SUBROUTINE ObjFunction
